#include <stdio.h>

/*SUDOKU_SOLVER
by iKar
v následujících komentářích budou použity výrazy skupina: značí pole 9 int a čtverec myšleno 3x3 políčka čtverec v sudoku */

//konstanty pro SudokuToNum()
#define INCORECT_INPUT -1	//nezpracovatelný vstup
#define EMPTY 0			//prázdné pole
#define TONUM 48		//rozdíl mezi číslem a hodnotou číslovky v ASCII tabulce znaků

//výčtový typ pro options[9][3] použité při hledání možných pozic čísla NUM je číslo, LINE řádek možné pozice a COLUMN sloupec
typedef enum{
	NUM,
	LINE,
	COLUMN
}OPTIONS;

//funkce převádí znaky na čísla v případě prázdného místa na 0
int SudokuToNum(char c)
{
	if((c >= '1') && (c <= '9')){
		return (c-TONUM);
	}
	if(c == 95){
		return EMPTY;
	}
	else {
		return INCORECT_INPUT;
	}
}

char SudokuGetChar(void)
{
	char c;
	//cyklus čekající na očekávané znaky
	while((c = getchar()) != '\n' && c != 95 && !(c > '1' || c < '9'));
	return c;
}

//načte sudoku do pole při neúspěchu vrací 1 jinak 0
int SudokuOpen(int sudoku[9][9])
{
	char inputc;		//vstupní znak
	int numLine;		//číslo řádku pro načítací cyklus
	int numColumn;		//číslo sloupce pro načítací cyklus

	for(numLine = 0; numLine < 9; numLine++){
		for(numColumn = 0; numColumn < 9; numColumn++){
			inputc = SudokuGetChar();
			if((sudoku[numLine][numColumn] = SudokuToNum(inputc)) == INCORECT_INPUT){
				printf("chybný vstup\n");
				return 1;
			}
			if((inputc = getchar()) != ' ' && numColumn < 8){
				printf("chybný vstup, (vynechaná mezera)\n");
				return 1;
			}
		}
	}
	return 0;
}

//uloží do pole line[9] numLine-tý řádek
void SudokuGetLine(int sudoku[9][9], int numLine, int line[9])
{
	int numColumn;	//číslo sloupce pro cyklus 

	for(numColumn = 0; numColumn < 9; numColumn++){
		line[numColumn] = sudoku[numLine][numColumn];
	}
	return;
}

//uloží do pole column[9] numColumn-tý sloupec
void SudokuGetColumn(int sudoku[9][9], int numColumn, int column[9])
{
	int numLine;	//číslo sloupce pro cyklus

	for(numLine = 0; numLine < 9; numLine++){
		column[numLine] = sudoku[numLine][numColumn];
	}
	return;
}

void SudokuGetSector(int sudoku[9][9], int numsector, int sector[9])
{
	int numLine, numColumn;
	int columnPlus, linePlus;

	linePlus = (((numsector)/3)*3);		//určuje o kolik je čtverec posunutý vzhledem k levému hornímu rohu ve svislém směru
	columnPlus = (((numsector)%3)*3);	//určuje o kolik je čtverec posunutý vzhledem k levému hornímu rohu ve vodorovném směru
	
	//tyto dva cykly projdou čtverec a nakopírují ho do sector[9]
	for(numLine = 0; numLine < 3; numLine++){
		for(numColumn = 0; numColumn < 3; numColumn++){
			sector[(numLine*3)+numColumn] = sudoku[numLine+linePlus][numColumn+columnPlus];
		}
	}
	return;
}

//funkce vypočte v jakém se nachází čtverci pole na průsečíku column a line
int SudokuGetPosSector(int line, int column)
{
	line/=3;
	column/=3;
	return 3*line+column;
}

//zjišťuje jestli je v dané skupině číslo num když ano vrací 1 jinak 0
int SudokuIsIn(int num, int group[9])
{
	int i;	//index pro projití pole
	
	for(i = 0; i < 9; i++){
		if(group[i] == num){
			return 0;
		}
	}
	return 1;
}

//Zjišťuje která čísla od 1 do 9 chybí ve group a uloží je do missing
//vrací počet chybících čísel
int SudokuGetMissing(int group[9], int missing[9])
{
	int testNum;		//proměná pro testovaná čísla
	int iMissing = 0;	//index pro pole missing
	for(testNum = 1; testNum <= 9; testNum++){
		if(SudokuIsIn(testNum, group) == 1){
			missing[iMissing]=testNum;
			iMissing++;
		}
	}
	return iMissing;
}

//zjistí která pole jsou ve skupině prázdná a uloží jejich index do empty
//vrací počet prázdných polí
int SudokuGetEmpty(int group[9], int empty[9])
{
	int iGroup;			//index pro pole group
	int iEmpty = 0;			//index pro pole empty
	for(iGroup = 0; iGroup < 9; iGroup++){
		if(group[iGroup] == 0){
			empty[iEmpty]=iGroup;
			iEmpty++;
		}
	}
	return iEmpty;
}

//přepíše každý prvek skupiny číslem numToWrite
void SudokuClearGroup(int group[9], int numToWrite)
{
	int iGroup;

	for(iGroup = 0; iGroup <= 9; iGroup++){
		group[iGroup]=numToWrite;
	}
	return;
}

//zjišťuje jestli jsou všechna pole sudoku doplněná pokud ano vrací 0 jinak 1
int IsSudokuComplete(int sudoku[9][9])
{
	int numLine;

	for(numLine = 0; numLine < 9; numLine++){
		if(SudokuIsIn(0, sudoku[numLine]) == 0){
			return 1;
		}
	}
	return 0;
}

//funkce vyhledává v numLine-tém řádku čísle která je možné zapsat pouze na jednu pozici
//pokud je to možné zapíše ho tam
//vrací počet doplněných polí, pokud některé číslo nejde umístit na žádnou pozici vrací -1 (je neřešitelné)
int SudokuLineSolver(int sudoku[9][9], int numLine)
{
	int testedNum;		//proměnná pro testované číslo
	int numColumn;		//proměnná pro číslo sloupce (neboli pozice pole v řádku)
	int option;		//proměnná do které se zaznamenává počet moných pozic čísla
	int optionColumn;	//proměnná do které se ukládá možná poice čísla (sloupec)
	int answer = 0;		//proměnná do které se zaznamenává počet vyřešených polí
	int line[9];		//pole do kterého se načte řádek
	int column[9];		//pole do kterého se načte sloupec
	int sector[9];		//pole do kterého se načte čtverec
	int empty[9];		//pole do kterého se ukládájí indexy prázdných polí
	int missing[9];		//pole do kterého se ukládají chybějící čísla

	SudokuClearGroup(empty, -1);			//incializace
	SudokuClearGroup(missing, 0);			//inicializace
	SudokuGetLine(sudoku, numLine, line);		//načtení řádku
	SudokuGetMissing(line, missing);		//zjištění chybějících čísel
	SudokuGetEmpty(line, empty);			//zjištění prázdných pozic

	//cyklus procházející chybějící čísla
	for(testedNum = 1; testedNum <= 9; testedNum++){
		option = 0;					//vynulování počtu možných pozic
		//test jestli číslo chybí
		if(SudokuIsIn(testedNum, missing) == 1){
			continue;
		}
		//cyklus procházející sloupce
		for(numColumn = 0; numColumn < 9; numColumn++){
			//zjištění jestli je pozice volná
			if(SudokuIsIn(numColumn, empty) == 1){
				continue;
			}
			SudokuGetColumn(sudoku, numColumn, column);					//načtení příslušného sloupce
			SudokuGetSector(sudoku, SudokuGetPosSector(numLine, numColumn), sector);	//načtení příslužného čtverce
			//zkouška jestli se testedNum nenachází v příslušném sloupci nebo čtverci
			if(((SudokuIsIn(testedNum, column) == 1) && (SudokuIsIn(testedNum, sector) == 1))){
				option++;			//zvětšení počtu možných pozic o 1
				optionColumn = numColumn;	//uložení možné pocice
			}
		}
		//podmínka pro pouze jednu možnou pozici
		if(option == 1){
			sudoku[numLine][optionColumn]=testedNum;	//zapsání čísla do sudoku
			answer++;					//zvětšení počtu vyřešených polí o 1
		}
		//podmínka pro žádnou potencionální pozici
		else if(option == 0){
			return -1;
		}
	}
	return answer;
}


//funkce vyhledává v numColumn-tém sloupci čísla která je možné zapsat pouze na jednu pozici
//pokud je to možné zapíše ho tam
//vrací počet doplněných polí, pokud některé číslo nejde umístit na žádnou pozici vrací -1 (je neřešitelné)
//funkce je prakticky stejná jako u SudokuLineSolver()
int SudokuColumnSolver(int sudoku[9][9], int numColumn)
{

	int testedNum;
	int numLine;
	int option;
	int optionLine;
	int answer = 0;
	int line[9];
	int column[9];
	int sector[9];
	int empty[9];
	int missing[9];

	SudokuClearGroup(empty, -1);
	SudokuClearGroup(missing, 0);
	SudokuGetColumn(sudoku, numColumn, column);
	SudokuGetMissing(column, missing);
	SudokuGetEmpty(column, empty);
	for(testedNum = 1; testedNum <= 9; testedNum++){
		option = 0;
		if(SudokuIsIn(testedNum, missing) == 1){
			continue;
		}
		for(numLine = 0; numLine < 9; numLine++){
			if(SudokuIsIn(numLine, empty) == 1){
				continue;
			}
			SudokuGetLine(sudoku, numLine, line);
			SudokuGetSector(sudoku, SudokuGetPosSector(numLine, numColumn), sector);
			if(((SudokuIsIn(testedNum, line) == 1) && (SudokuIsIn(testedNum, sector) == 1))){
				option++;
				optionLine = numLine;
			}
		}
		if(option == 1){
			sudoku[optionLine][numColumn]=testedNum;
			answer++;
		}
		else if(option == 0){
			return -1;
		}
	}
	return answer;
}

//funkce vyhledává v numSector-tém čtverc čísla která je možné zapsat pouze na jednu pozici
//pokud je to možné zapíše ho tam
//vrací počet doplněných polí, pokud některé číslo nejde umístit na žádnou pozici vrací -1 (je neřešitelné)
//funkce je prakticky stejná jako u SudokuLineSolver()
int SudokuSectorSolver(int sudoku[9][9], int numSector)
{

	int testedNum;
	int posInSector;
	int optionPosInSector;
	int option;
	int answer = 0;
	int line[9];
	int column[9];
	int sector[9];
	int empty[9];
	int missing[9];

	SudokuClearGroup(empty, -1);
	SudokuClearGroup(missing, 0);
	SudokuGetSector(sudoku, numSector, sector);
	SudokuGetMissing(sector, missing);
	SudokuGetEmpty(sector, empty);
	for(testedNum = 1; testedNum <= 9; testedNum++){
		option = 0;
		if(SudokuIsIn(testedNum, missing) == 1){
			continue;
		}
		for(posInSector = 0; posInSector < 9; posInSector++){
			if(SudokuIsIn(posInSector, empty) == 1){
				continue;
			}
			SudokuGetColumn(sudoku, (((numSector%3)*3)+(posInSector%3)), column);
			SudokuGetLine(sudoku, (((numSector/3)*3)+(posInSector/3)), line);
			if(((SudokuIsIn(testedNum, line) == 1) && (SudokuIsIn(testedNum, column) == 1))){
				option++;
				optionPosInSector = posInSector;
			}
		}
		if(option == 1){
			sudoku[(((numSector/3)*3)+(optionPosInSector/3))][(((numSector%3)*3)+(optionPosInSector%3))]=testedNum;
			answer++;
		}
		else if(option == 0){
			return -1;
		}
	}
	return answer;
}

//hledá možná umístění jednoho čísla začína na numLine-tém počtu možností a končí na 9
int SudokuSearchOptionsInLine(int sudoku[9][9], int options[9][3], int numOptions, int numLine){
	int line[9], column[9], sector[9];
	int empty[9], missing[9];
	int testedNum;
	int numColumn;
	int option = 0;
	int i;			//index pro pole options


	SudokuClearGroup(line, 0);				//inicializace
	SudokuClearGroup(empty, -1);				//inicializace
	SudokuClearGroup(missing, 0);				//inicializace
	SudokuGetLine(sudoku, numLine, line);			//načtení řádku
	SudokuGetEmpty(line, empty);				//zjištěný prázdných polí
	SudokuGetMissing(line, missing);			//zjištění chybýcích čísel
	//cyklus prochází čísla od 1 do 9
	for(testedNum = 1; testedNum <= 9; testedNum++){
		//kontrola jestli číslo chybí
		if(SudokuIsIn(testedNum, missing) == 1){
			continue;
		}
		//cyklus procházející sloupce
		for(numColumn = 0; numColumn < 9; numColumn++){
			//kontrola jestli je pole prázdné
			if(SudokuIsIn(numColumn, empty) == 1){
				continue;
			}
			//načtení čtverce a sloupce
			SudokuClearGroup(column, 0);
			SudokuGetColumn(sudoku, numColumn, column);
			SudokuClearGroup(sector, 0);
			SudokuGetSector(sudoku, SudokuGetPosSector(numLine, numColumn), sector);
			//kontrola jestli číslo není ve sloupci nebo čtverci
			if((SudokuIsIn(testedNum, column) == 1) && (SudokuIsIn(testedNum, sector) == 1)){
				//zapsání možné pozice čísla
				options[option][NUM] = testedNum;
				options[option][LINE] = numLine;
				options[option][COLUMN] = numColumn;
				option++;				//zvětšení počtu možností o 1
				//pokud je překročen počet možností je pole options vynulováno a proměnná option též
				if(option > numOptions){
					for(i = 0; i <= option; i++){
						options[i][NUM] = 0;
						options[i][LINE] = 0;
						options[i][COLUMN] = 0;
					}
					option = 0;
					break;
				}
			}
		}
		if(option == numOptions){
			//nulování nevyužité částli pole options
			for(i = option; i < 9; i++){
				options[i][NUM]=0;
			}
			return 0;
		}
		else if(option < numOptions){
			option = 0;
		}
	}
	return 1;
}

//Velice podobné SudokuSearchOptionsInLine
int SudokuSearchOptionsInColumn(int sudoku[9][9], int options[9][3], int numOptions, int numColumn)
{
	int line[9], column[9], sector[9];
	int empty[9], missing[9];
	int testedNum;
	int numLine;
	int option = 0;
	int i;

	SudokuClearGroup(column, 0);
	SudokuClearGroup(empty, -1);
	SudokuClearGroup(missing, 0);
	SudokuGetColumn(sudoku, numColumn, column);
	SudokuGetEmpty(column, empty);
	SudokuGetMissing(column, missing);
	for(testedNum = 1; testedNum <= 9; testedNum++){
		if(SudokuIsIn(testedNum, missing) == 1){
			continue;
		}
		for(numLine = 0; numLine < 9; numLine++){
			if(SudokuIsIn(numLine, empty) == 1){
				continue;
			}
			SudokuClearGroup(line, 0);
			SudokuGetLine(sudoku, numLine, line);
			SudokuClearGroup(sector, 0);
			SudokuGetSector(sudoku, SudokuGetPosSector(numLine, numColumn), sector);
			if((SudokuIsIn(testedNum, line) == 1) && (SudokuIsIn(testedNum, sector) == 1)){
				options[option][NUM] = testedNum;
				options[option][LINE] = numLine;
				options[option][COLUMN] = numColumn;
				option++;
				if(option > numOptions){
					for(i = 0; i <= option; i++){
						options[i][NUM] = 0;
						options[i][LINE] = 0;
						options[i][COLUMN] = 0;
					}
					option = 0;
					break;
				}
			}
		}
		if(option == numOptions){
			for(i = option; i < 9; i++){
				options[i][NUM]=0;
			}
			return 0;
		}
		else if(option < numOptions){
			option = 0;
		}
	}
	return 1;
}

//Velice podobné SudokuSearchOptionsInLine
int SudokuSearchOptionsInSector(int sudoku[9][9], int options[9][3], int numOptions, int numSector)
{
	int line[9], column[9], sector[9];
	int empty[9], missing[9];
	int testedNum;
	int posInSector;
	int option = 0;
	int i;

	SudokuClearGroup(sector, 0);
	SudokuClearGroup(empty, -1);
	SudokuClearGroup(missing, 0);
	SudokuGetSector(sudoku, numSector, sector);
	SudokuGetEmpty(sector, empty);
	SudokuGetMissing(sector, missing);
	for(testedNum = 1; testedNum <= 9; testedNum++){
		if(SudokuIsIn(testedNum, missing) == 1){
			continue;
		}
		for(posInSector = 0; posInSector < 9; posInSector++){
			if(SudokuIsIn(posInSector, empty) == 1){
				continue;
			}
			SudokuClearGroup(line, 0);
			SudokuGetLine(sudoku, (((numSector/3)*3)+(posInSector/3)), line);
			SudokuClearGroup(column, 0);
			SudokuGetColumn(sudoku, (((numSector%3)*3)+(posInSector%3)), column);
			if((SudokuIsIn(testedNum, line) == 1) && (SudokuIsIn(testedNum, column) == 1)){
				options[option][NUM] = testedNum;
				options[option][LINE] = (((numSector/3)*3)+(posInSector/3));
				options[option][COLUMN] = (((numSector%3)*3)+(posInSector%3));
				option++;
				if(option > numOptions){
					for(i = 0; i <= option; i++){
						options[i][NUM] = 0;
						options[i][LINE] = 0;
						options[i][COLUMN] = 0;
					}
					option = 0;
					break;
				}
			}
		}
		if(option == numOptions){
			for(i = option; i < 9; i++){
				options[i][NUM]=0;
			}
			return 0;
		}
		else if(option < numOptions){
			option = 0;
		}
	}
	return 1;
}

//Spojuje 3 předchozí funkce
int SudokuSearchOptions(int sudoku[9][9], int options[9][3], int numOptions)
{
	int numLine, numColumn, numSector;
	int answer;

	//co je prázdné? co chybí? cyklus 1  9 když je na prázdném možné 2x 
	for(numOptions; numOptions <= 9; numOptions++){
		for(numLine = 0; numLine < 9; numLine++){
			if(SudokuSearchOptionsInLine(sudoku, options, numOptions, numLine) == 0){
				return numOptions;
			}
		}
		for(numColumn = 0; numColumn < 9; numColumn++){
			if(SudokuSearchOptionsInColumn(sudoku, options, numOptions, numColumn) == 0){
				return numOptions;
			}
		}
		for(numSector = 0; numSector < 9; numSector++){
			if(SudokuSearchOptionsInSector(sudoku, options, numOptions, numSector) == 0){
				return numOptions;
			}
		}
	} 

	return 0;
}

//spojuje a ošetřuje SudokuLine/Column/SectorSolver() vrací 1 při neřešitelné situaci jinak počet vyřešených polí
int SudokuSolverHuman(int sudoku[9][9]){
	int answer, answerTest;
	int numLine, numColumn, numSector;
	int options[9][3];
	int numOptions;
	int sudokuSwap[9][9];
	int option;

	answer = 1;
	while(answer > 0){
	answer = 0;
		for(numLine = 0; numLine < 9; numLine++){
			if((answerTest=SudokuLineSolver(sudoku, numLine)) == -1){
				return 1;
			}
			else{
				answer+=answerTest;
			}
		}
		for(numColumn = 0; numColumn < 9; numColumn++){
			if((answerTest=SudokuColumnSolver(sudoku, numColumn)) == -1){
				return 1;
			}
			else{
				answer+=answerTest;
			}
		}
		for(numSector = 0; numSector < 9; numSector++){
			if((answerTest=SudokuSectorSolver(sudoku, numSector)) == -1){
				return 1;
			}
			else{
				answer+=answerTest;
			}
		}
	}
	return 0;
}

//volá SudokuSolverHuman a pokud sudoku nedořeší tak dosadí číslo na potenciální umístění a zavolá sama sebe s dosazeným číslem
//vrací 1 když nastane neřešitelná situace jinak 0
int SudokuSolver(int sudoku[9][9])
{
	int numOptions, numLine, numColumn;
	int option;
	int options[9][3];
	int swapSudoku[9][9];

	if(SudokuSolverHuman(sudoku) == 1){
		return 1;
	}
	while(IsSudokuComplete(sudoku) != 0){
		numOptions = SudokuSearchOptions(sudoku, options, 2);
		for(option = 0; option < numOptions; option++){
			for(numLine = 0; numLine < 9; numLine++){
				for(numColumn = 0; numColumn < 9; numColumn++){
					swapSudoku[numLine][numColumn] = sudoku[numLine][numColumn];
				}
			}
			swapSudoku[options[option][LINE]][options[option][COLUMN]] = options[option][NUM];
			if(SudokuSolver(swapSudoku) == 0){
				for(numLine = 0; numLine < 9; numLine++){
					for(numColumn = 0; numColumn < 9; numColumn++){
						sudoku[numLine][numColumn] = swapSudoku[numLine][numColumn];
					}
				}
				break;
			}
		}
	}
	return 0;
}
